==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webmail/client/inc/httprequest.js <==
/*
   IceWarp Webmail v.6
   XML
*/


/************* cRequest ************  !! poresit sResponse vs bSync
 * @brief   Build HTTP Request Obj.
 * @author  DRZ 18.05.2005
 * @require:    XMLTools
 *
------------------------------
 pokusit se udelat this.oXMLHttp jako static

 * Properties:
 *    + sURL
 *    + bSync
 *    + bGET
 *
 *  Methods:
 *    + sendXML(xml[, replylistener])
 *    + sendArr(array[, replylistener])
 *    + sendStr(text[, replylistener])
 *    + this.oXMLHttp.responseText
 *    + this.oXMLHttp.responseXML
 *    + this.oXMLHttp.responseArr
 */
/*
  this.oXMLHttp.abort();

*/

function cHttpRequest(){
	this.sURL = './server/webmail.php';	// Default request URL
	this.bSync = false;					// false - async, true - sync
	//this.bGET = false;				// request type - GET|POST
	this.bIE = false;
	this.oXMLHttp = null;

	// prepare request Obj.
	if (!this.oXMLHttp){
		if (window.XMLHttpRequest){
			this.oXMLHttp = new XMLHttpRequest();

			//this.oXMLHttp.overrideMimeType('text/xml'); - potlaci nutnost XML hlavicky
			
			//MSIE 7
			this.bIE = currentBrowser().indexOf('MSIE')>-1;
		}
		else
		//MSIE 6
		if(!navigator.__ice_version && window.ActiveXObject){
			this.oXMLHttp = new ActiveXObject('Microsoft.XMLHTTP');
			this.bIE = true;
		}
		else
			throw new Error("cHTTPRequest() - Your browser does not support XMLHttpRequest objects");
	}
};

	/**
	 * @brief:	Send XML document thru HTTPRequest
	 * @param:	xData     - XML document
	 *			oResponse - response object [object,method,[optional params,...]]
	 * @date :	26.5.2006 16:37:19
	 **/
	cHttpRequest.prototype.sendXML = function(xData,oResponse) {
		var oXMLHttp = this.oXMLHttp;
        var bSync = (typeof oResponse == 'object')?false:true;
		var bGET = xData?false:true;

		oXMLHttp.open((bGET?'GET':'POST'), this.sURL, !bSync);
		oXMLHttp.send(xData?xData:null);

		//sync mode
		if(this.bSync) {
			if (oXMLHttp.status && oXMLHttp.status!=200 && oXMLHttp.status!=304)
				throw new Error("cHTTPRequest.send() - sync - URL "+this.sURL+"\n returned: "+ oXMLHttp.statusText + " [" + oXMLHttp.status + "]");
			return;
		}

		var me = this;

		//async mode
		oXMLHttp.onreadystatechange = function () {

			if (oXMLHttp.readyState == 4) {

				try{
					if (oXMLHttp.status && oXMLHttp.status!=200 && oXMLHttp.status!=304){
						throw new Error("cHTTPRequest.send() - async - URL "+this.sURL+"\n returned "+ oXMLHttp.statusText + " [" + oXMLHttp.status + "]");
						return;
					}
				}
				catch(er){
					return;
				}

				if (Is.Array(oResponse)){

					try{
						switch(oResponse[3]){
						case 'XML':
							var oRData = {"XML":me.responseXML()};
							break;
						case 'Text':
							var oRData = {"Text":me.responseText()};
							break;
						default:
							var oRData = {"Array":me.responseArray()};
							if (oRData.Array.PARSERERROR){
								if (oRData.Array.PARSERERROR[0])
									alert("oResponse Error: \n"+oRData.Array.PARSERERROR[0].VALUE +"\n"+me.responseText());
							}
						}
					}
					catch(err){
						alert("oResponse Error: \n"+me.responseText());
					}

					executeCallbackFunction(oResponse, oRData);
				}
				else{
					try{
						var oRData = {"Text":me.responseText(),"Array":me.responseArray()};
					}
					catch(err){
						alert("Can't convert into Array: \n" + me.responseText());
					}

					/* oResponse as object, create obj.responseData = dat and call obj.response() method */
					try {
						oResponse.prototype.responseData = oRData;
						oResponse.response();
					}
					catch(e) {
						throw new Error("cHTTPRequest.send() -  URL "+this.sURL+"\n erron in "+oResponse.toString());
					}
				}
				oRData = null;
				oResponse = null; //Response destruct
			}
		};
	};

    /**
     * @brief: strip WhiteSpaces from XML
     */
    cHttpRequest.prototype.stripWhiteSpace = function (node) {
            nodesToDelete = Array();
            this.findWhiteSpace(node, 0);
            for(var i=nodesToDelete.length-1;i>=0;i--) {
                    nodeRef = nodesToDelete[i];
                    nodeRef.parentNode.removeChild(nodeRef);
            }
    };
          cHttpRequest.prototype.is_ws = function (nod) {
                  return !(/[^\t\n\r ]/.test(nod.data));
          };

          cHttpRequest.prototype.findWhiteSpace = function (node, nodeNo) {
                  for (var i=0; i<node.childNodes.length; i++) {
                          if (node.childNodes[i].nodeType == 3 && this.is_ws(node.childNodes[i]))
                             nodesToDelete[nodesToDelete.length] = node.childNodes[i];
                          if (node.childNodes[i].hasChildNodes())
                             this.findWhiteSpace(node.childNodes[i], i);
                  }
                  node = node.parentNode;
                  i = nodeNo;
          };

    // PUBLIC
    cHttpRequest.prototype.sendArray = function(aData,oResponse){
         /* DEBUG */
         if(IWAPI_DEBUG) api_debug.add('send Array',aData);

//		 var xml = XMLTools.Arr2XML(aData); inspect2({ARRAY:aData,"XML":typeof xml, "STRING":XMLTools.XML2Str(xml)});

		 this.sendXML(XMLTools.Arr2XML(aData),oResponse);
    };
    cHttpRequest.prototype.sendString = function(sData,oResponse){
       /* DEBUG */
       if(IWAPI_DEBUG) api_debug.add('send String',sData);

       this.sendXML(XMLTools.Str2XML(sData),oResponse);
    };
    cHttpRequest.prototype.responseText = function(){
       var s = this.oXMLHttp.responseText;

       /* DEBUG */
       if(IWAPI_DEBUG) api_debug.add('response String',s);
       
       return s;
    };
	cHttpRequest.prototype.responseXML = function(){

	var xml = this.oXMLHttp.responseXML;

	// Strip white space from response when browser isn't IE
	if (!this.bIE) {
		var child = xml.childNodes[0];
		if (child && child.hasChildNodes && child.childNodes[0].nodeType == 3 && this.is_ws(child.childNodes[0]))
			this.stripWhiteSpace(xml);
	}

	return xml;
};

	cHttpRequest.prototype.responseArray = function(){

		var a;
		try{
			if (this.oXMLHttp.getResponseHeader("Content-Type") == 'text/json')
			    a = new Function("return "+this.responseText())();
			else
				a = XMLTools.XML2Arr(this.responseXML());
		}
		catch(e){
		  inspect2({"ERROR XML->ARRAY":this.responseText()},1);
		}

		/* DEBUG */
		if(IWAPI_DEBUG) api_debug.add('response Array',a);

		return a;
    };

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webmail/client/inc/httprequest.js <==
/*
   IceWarp Webmail v.6
   XML
*/


/************* cRequest ************  !! poresit sResponse vs bSync
 * @brief   Build HTTP Request Obj.
 * @author  DRZ 18.05.2005
 * @require:    XMLTools
 *
------------------------------
 pokusit se udelat this.oXMLHttp jako static

 * Properties:
 *    + sURL
 *    + bSync
 *    + bGET
 *
 *  Methods:
 *    + sendXML(xml[, replylistener])
 *    + sendArr(array[, replylistener])
 *    + sendStr(text[, replylistener])
 *    + this.oXMLHttp.responseText
 *    + this.oXMLHttp.responseXML
 *    + this.oXMLHttp.responseArr
 */
/*
  this.oXMLHttp.abort();

*/

function cHttpRequest(){
	this.sURL = 'server/webmail.php';	// Default request URL
	//this.bSync = false;					// false - async, true - sync
	//this.bGET = false;				// request type - GET|POST
	this.bIE = false;
	this.oXMLHttp = null;

	// prepare request Obj.
	if (!this.oXMLHttp){
		if (window.XMLHttpRequest){
			this.oXMLHttp = new XMLHttpRequest();

			//MSIE 7
			this.bIE = currentBrowser().indexOf('MSIE')>-1;
		}
		else
		//MSIE 6
		if(!navigator.__ice_version && window.ActiveXObject){
			this.oXMLHttp = new ActiveXObject('Microsoft.XMLHTTP');
			this.bIE = true;
		}
		else
			throw new Error("cHTTPRequest() - Your browser does not support XMLHttpRequest objects");
	}
};

	/**
	 * @brief:	Send XML document thru HTTPRequest
	 * @param:	xData     - XML document
	 *			oResponse - response object [object,method,[optional params,...]]
	 * @date :	26.5.2006 16:37:19
	 **/
	cHttpRequest.prototype.sendXML = function(xData,oResponse) {
		var oXMLHttp = this.oXMLHttp;
        var bSync = (typeof oResponse == 'object')?false:true;
		var bGET = xData?false:true;

		if (!bSync && !bGET){
			gui._loading(true);
        }
        
		oXMLHttp.open((bGET?'GET':'POST'), this.sURL, !bSync);

/*
		if (xData){
			oXMLHttp.setRequestHeader('Content-Type','application/xml;charset=utf-8');
			oXMLHttp.setRequestHeader("encoding", "UTF-8");
		}
*/
		oXMLHttp.send(xData?xData:null);

		//clear memory
		xData = null;
		
		//sync mode
		if(bSync) {
			if (oXMLHttp.status && oXMLHttp.status!=200 && oXMLHttp.status!=304)
				throw new Error("cHTTPRequest.send() - sync - URL "+this.sURL+"\n returned: "+ oXMLHttp.statusText + " [" + oXMLHttp.status + "]");

			return;
		}

		var me = this;

		//async mode
		oXMLHttp.onreadystatechange = function () {
			try{
				if (oXMLHttp.readyState == 4) {

					if (oXMLHttp.status && oXMLHttp.status!=200 && oXMLHttp.status!=304){
						throw "cHTTPRequest.send() - async - URL "+this.sURL+"\n returned "+ oXMLHttp.statusText + " [" + oXMLHttp.status + "]";
						return;
					}

					if (Is.Array(oResponse)){

						try{
							switch(oResponse[3]){
							case 'XML':
								var oRData = {"XML":me.responseXML()};
								break;
							case 'Text':
								var oRData = {"Text":me.responseText()};
								break;
							default:
								var oRData = {"Array":me.responseArray()};
								if (oRData.Array.PARSERERROR){
									if (oRData.Array.PARSERERROR[0])
										alert("oResponse Error: \n"+oRData.Array.PARSERERROR[0].VALUE +"\n"+me.responseText());
								}
							}
						}
						catch(err){
							alert("oResponse Error: \n"+me.responseText());
						}

						executeCallbackFunction(oResponse, oRData);
					}
					else{
						try{
							var oRData = {"Text":me.responseText(),"Array":me.responseArray()};
						}
						catch(err){
							alert("Can't convert into Array: \n" + me.responseText());
						}

						/* oResponse as object, create obj.responseData = dat and call obj.response() method */
						try {
							oResponse.prototype.responseData = oRData;
							oResponse.response();
						}
						catch(e) {
							alert("cHTTPRequest.send() -  URL "+this.sURL+"\n erron in "+oResponse.toString());
						}
					}
					oRData = null;
					oResponse = null; //Response destruct

					if (!bGET) gui._loading();
				}
			}
			catch(e){

				if (!bGET) gui._loading();
				
				if (Is.Array(oResponse)){
					executeCallbackFunction(oResponse,{"Array":{IQ:[{ATTRIBUTES:{TYPE:'error'},ERROR:[{ATTRIBUTES:{UID:'offline'},VALUE:'Connection Lost'}]}]}});
					oResponse = null;
				}
			}
		};
	};

    /**
     * @brief: strip WhiteSpaces from XML
     */
    cHttpRequest.prototype.stripWhiteSpace = function (node) {
            nodesToDelete = Array();
            this.findWhiteSpace(node, 0);
            for(var i=nodesToDelete.length-1;i>=0;i--) {
                    nodeRef = nodesToDelete[i];
                    nodeRef.parentNode.removeChild(nodeRef);
            }
    };
          cHttpRequest.prototype.is_ws = function (nod) {
                  return !(/[^\t\n\r ]/.test(nod.data));
          };

          cHttpRequest.prototype.findWhiteSpace = function (node, nodeNo) {
                  for (var i=0; i<node.childNodes.length; i++) {
                          if (node.childNodes[i].nodeType == 3 && this.is_ws(node.childNodes[i]))
                             nodesToDelete[nodesToDelete.length] = node.childNodes[i];
                          if (node.childNodes[i].hasChildNodes())
                             this.findWhiteSpace(node.childNodes[i], i);
                  }
                  node = node.parentNode;
                  i = nodeNo;
          };

    // PUBLIC
    cHttpRequest.prototype.sendArray = function(aData,oResponse){
         /* DEBUG */
         if(IWAPI_DEBUG) api_debug.add('send Array',aData);
		 this.sendXML(XMLTools.Arr2XML(aData),oResponse);
    };
    cHttpRequest.prototype.sendString = function(sData,oResponse){
       /* DEBUG */
       if(IWAPI_DEBUG) api_debug.add('send String',sData);

       this.sendXML(XMLTools.Str2XML(sData),oResponse);
    };
    cHttpRequest.prototype.responseText = function(){
       var s = this.oXMLHttp.responseText;

       /* DEBUG */
       if(IWAPI_DEBUG) api_debug.add('response String',s);
       
       return s;
    };
	cHttpRequest.prototype.responseXML = function(){

	var xml = this.oXMLHttp.responseXML;

	// Strip white space from response when browser isn't IE
	if (!this.bIE) {
		var child = xml.childNodes[0];
		if (child && child.hasChildNodes && child.childNodes[0].nodeType == 3 && this.is_ws(child.childNodes[0]))
			this.stripWhiteSpace(xml);
	}

	return xml;
};

	cHttpRequest.prototype.responseArray = function(){

		var a;
		try{
			if (this.oXMLHttp.getResponseHeader("Content-Type") == 'text/json')
			    a = new Function("return "+this.responseText())();
			else
				a = XMLTools.XML2Arr(this.responseXML());
		}
		catch(e){
		  inspect2({"ERROR XML->ARRAY":this.responseText()},1);
		}

		/* DEBUG */
		if(IWAPI_DEBUG) api_debug.add('response Array',a);

		return a;
    };
